@@ -18,6 +18,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -28,10 +31,6 @@
 
 #include "wocky-test-sasl-auth-server.h"
 
-#include <wocky/wocky-xmpp-connection.h>
-#include <wocky/wocky-utils.h>
-#include <wocky/wocky-namespaces.h>
-#include "config.h"
 
 #ifdef HAVE_LIBSASL2
 
