@@ -28,16 +28,19 @@
  * XEP-0004.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "wocky-data-form.h"
 
 #include <string.h>
 
-#include "wocky-data-form-enumtypes.h"
 #include "wocky-namespaces.h"
 #include "wocky-utils.h"
 
-#define DEBUG_FLAG DEBUG_DATA_FORM
-#include "wocky-debug.h"
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_DATA_FORM
+#include "wocky-debug-internal.h"
 
 G_DEFINE_TYPE (WockyDataForm, wocky_data_form, G_TYPE_OBJECT)
 
