@@ -19,6 +19,10 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -28,10 +32,9 @@
 #include "wocky-signals-marshal.h"
 #include "wocky-namespaces.h"
 #include "wocky-utils.h"
-#include "wocky-xmpp-error-enumtypes.h"
 
-#define DEBUG_FLAG DEBUG_AUTH
-#include "wocky-debug.h"
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_AUTH
+#include "wocky-debug-internal.h"
 
 G_DEFINE_TYPE(WockySaslAuth, wocky_sasl_auth, G_TYPE_OBJECT)
 
@@ -290,6 +293,7 @@ static gboolean
 stream_error (WockySaslAuth *sasl, WockyStanza *stanza)
 {
   WockyStanzaType type = WOCKY_STANZA_TYPE_NONE;
+  WockySaslAuthPrivate *priv = sasl->priv;
 
   if (stanza == NULL)
     {
@@ -313,6 +317,14 @@ stream_error (WockySaslAuth *sasl, WockyStanza *stanza)
       return TRUE;
     }
 
+  if (g_cancellable_is_cancelled (priv->cancel))
+    {
+      /* We got disconnected but we still had this stanza to process. Don't
+       * bother with it. */
+      auth_failed (sasl, WOCKY_AUTH_ERROR_CONNRESET, "Disconnected");
+      return TRUE;
+    }
+
   return FALSE;
 }
 
@@ -332,27 +344,20 @@ wocky_sasl_auth_new (const gchar *server,
       NULL);
 }
 
-static gboolean
-each_mechanism (WockyNode *node, gpointer user_data)
-{
-  GSList **list = (GSList **)user_data;
-  if (wocky_strdiff (node->name, "mechanism"))
-    {
-      return TRUE;
-    }
-  *list = g_slist_append (*list, g_strdup (node->content));
-  return TRUE;
-}
-
 static GSList *
 wocky_sasl_auth_mechanisms_to_list (WockyNode *mechanisms)
 {
   GSList *result = NULL;
+  WockyNode *mechanism;
+  WockyNodeIter iter;
 
   if (mechanisms == NULL)
     return NULL;
 
-  wocky_node_each_child (mechanisms, each_mechanism, &result);
+  wocky_node_iter_init (&iter, mechanisms, "mechanism", NULL);
+  while (wocky_node_iter_next (&iter, &mechanism))
+    result = g_slist_append (result, g_strdup (mechanism->content));
+
   return result;
 }
 
@@ -610,7 +615,7 @@ wocky_sasl_auth_authenticate_finish (WockySaslAuth *sasl,
   GAsyncResult *result,
   GError **error)
 {
-  wocky_implement_finish_void (sasl, wocky_sasl_auth_authenticate_finish);
+  wocky_implement_finish_void (sasl, wocky_sasl_auth_authenticate_async);
 }
 
 static void
@@ -693,7 +698,7 @@ wocky_sasl_auth_authenticate_async (WockySaslAuth *sasl,
     }
 
   priv->result = g_simple_async_result_new (G_OBJECT (sasl),
-    callback, user_data, wocky_sasl_auth_authenticate_finish);
+    callback, user_data, wocky_sasl_auth_authenticate_async);
 
   if (cancellable != NULL)
     priv->cancel = g_object_ref (cancellable);
