@@ -1,3 +1,6 @@
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include "wocky-sasl-digest-md5.h"
 #include "wocky-auth-registry.h"
@@ -5,8 +8,8 @@
 
 #include <string.h>
 
-#define DEBUG_FLAG DEBUG_AUTH
-#include "wocky-debug.h"
+#define WOCKY_DEBUG_FLAG WOCKY_DEBUG_AUTH
+#include "wocky-debug-internal.h"
 
 typedef enum {
   WOCKY_SASL_DIGEST_MD5_STATE_STARTED,
