@@ -18,6 +18,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -26,13 +29,18 @@
 #include "wocky-stanza.h"
 #include "wocky-xmpp-error.h"
 #include "wocky-namespaces.h"
-#include "wocky-debug.h"
+#include "wocky-debug-internal.h"
+
+#include "wocky-node-private.h"
 
 G_DEFINE_TYPE(WockyStanza, wocky_stanza, WOCKY_TYPE_NODE_TREE)
 
 /* private structure */
 struct _WockyStanzaPrivate
 {
+  WockyContact *from_contact;
+  WockyContact *to_contact;
+
   gboolean dispose_has_run;
 };
 
@@ -122,6 +130,9 @@ wocky_stanza_init (WockyStanza *self)
 {
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, WOCKY_TYPE_STANZA,
       WockyStanzaPrivate);
+
+  self->priv->from_contact = NULL;
+  self->priv->to_contact = NULL;
 }
 
 static void wocky_stanza_dispose (GObject *object);
@@ -136,10 +147,9 @@ wocky_stanza_class_init (WockyStanzaClass *wocky_stanza_class)
 
   object_class->dispose = wocky_stanza_dispose;
   object_class->finalize = wocky_stanza_finalize;
-
 }
 
-void
+static void
 wocky_stanza_dispose (GObject *object)
 {
   WockyStanza *self = WOCKY_STANZA (object);
@@ -155,14 +165,29 @@ wocky_stanza_dispose (GObject *object)
     G_OBJECT_CLASS (wocky_stanza_parent_class)->dispose (object);
 }
 
-void
+static void
 wocky_stanza_finalize (GObject *object)
 {
+  WockyStanza *self = WOCKY_STANZA (object);
+
+  if (self->priv->from_contact != NULL)
+    {
+      g_object_unref (self->priv->from_contact);
+      self->priv->from_contact = NULL;
+    }
+
+  if (self->priv->to_contact != NULL)
+    {
+      g_object_unref (self->priv->to_contact);
+      self->priv->to_contact = NULL;
+    }
+
   G_OBJECT_CLASS (wocky_stanza_parent_class)->finalize (object);
 }
 
 WockyStanza *
-wocky_stanza_new (const gchar *name, const gchar *ns)
+wocky_stanza_new (const gchar *name,
+    const gchar *ns)
 {
   WockyStanza *result;
 
@@ -173,6 +198,18 @@ wocky_stanza_new (const gchar *name, const gchar *ns)
   return result;
 }
 
+WockyStanza *
+wocky_stanza_copy (WockyStanza *old)
+{
+  WockyNode *top;
+
+  top = _wocky_node_copy (wocky_stanza_get_top_node (old));
+
+  return g_object_new (WOCKY_TYPE_STANZA,
+      "top-node", top,
+      NULL);
+}
+
 static const gchar *
 get_type_name (WockyStanzaType type)
 {
@@ -208,23 +245,32 @@ get_sub_type_name (WockyStanzaSubType sub_type)
 
 static gboolean
 check_sub_type (WockyStanzaType type,
-                WockyStanzaSubType sub_type)
+    WockyStanzaSubType sub_type)
 {
+  WockyStanzaType expected_type;
+
   g_return_val_if_fail (type > WOCKY_STANZA_TYPE_NONE &&
       type < NUM_WOCKY_STANZA_TYPE, FALSE);
   g_return_val_if_fail (sub_type < NUM_WOCKY_STANZA_SUB_TYPE, FALSE);
 
   g_assert (sub_type_names[sub_type].sub_type == sub_type);
-  g_return_val_if_fail (
-      sub_type_names[sub_type].type == WOCKY_STANZA_TYPE_NONE ||
-      sub_type_names[sub_type].type == type, FALSE);
+
+  expected_type = sub_type_names[sub_type].type;
+
+  if (expected_type != WOCKY_STANZA_TYPE_NONE && expected_type != type)
+    {
+      g_critical ("Stanza sub-type '%s' may only be used with stanzas of "
+        "type '%s', not of type '%s'", sub_type_names[sub_type].name,
+        type_names[expected_type].name, type_names[type].name);
+      g_return_val_if_reached (FALSE);
+    }
 
   return TRUE;
 }
 
 static WockyStanza *
 wocky_stanza_new_with_sub_type (WockyStanzaType type,
-                                      WockyStanzaSubType sub_type)
+    WockyStanzaSubType sub_type)
 {
   WockyStanza *stanza = NULL;
   const gchar *sub_type_name;
@@ -250,7 +296,7 @@ wocky_stanza_new_with_sub_type (WockyStanzaType type,
  *           #WOCKY_STANZA_SUB_TYPE_SUBSCRIBED.)
  * @from: The sender's JID, or %NULL to leave it unspecified.
  * @to: The target's JID, or %NULL to leave it unspecified.
- * @Varargs: the description of the stanza to build,
+ * @...: the description of the stanza to build,
  *  terminated with %NULL
  *
  * Build a XMPP stanza from a list of arguments.
@@ -300,10 +346,10 @@ wocky_stanza_new_with_sub_type (WockyStanzaType type,
  */
 WockyStanza *
 wocky_stanza_build (WockyStanzaType type,
-                          WockyStanzaSubType sub_type,
-                          const gchar *from,
-                          const gchar *to,
-                          ...)
+    WockyStanzaSubType sub_type,
+    const gchar *from,
+    const gchar *to,
+    ...)
 
 {
   WockyStanza *stanza;
@@ -316,6 +362,32 @@ wocky_stanza_build (WockyStanzaType type,
   return stanza;
 }
 
+WockyStanza *
+wocky_stanza_build_to_contact (WockyStanzaType type,
+    WockyStanzaSubType sub_type,
+    const gchar *from,
+    WockyContact *to,
+    ...)
+
+{
+  WockyStanza *stanza;
+  va_list ap;
+  gchar *to_jid = NULL;
+
+  if (to != NULL)
+    to_jid = wocky_contact_dup_jid (to);
+
+  va_start (ap, to);
+  stanza = wocky_stanza_build_va (type, sub_type, from, to_jid, ap);
+  va_end (ap);
+
+  g_free (to_jid);
+
+  stanza->priv->to_contact = g_object_ref (to);
+
+  return stanza;
+}
+
 WockyStanza *
 wocky_stanza_build_va (WockyStanzaType type,
     WockyStanzaSubType sub_type,
@@ -389,8 +461,8 @@ get_sub_type_from_name (const gchar *name)
 
 void
 wocky_stanza_get_type_info (WockyStanza *stanza,
-                                  WockyStanzaType *type,
-                                  WockyStanzaSubType *sub_type)
+    WockyStanzaType *type,
+    WockyStanzaSubType *sub_type)
 {
   g_return_if_fail (stanza != NULL);
   g_assert (wocky_stanza_get_top_node (stanza) != NULL);
@@ -413,6 +485,7 @@ create_iq_reply (WockyStanza *iq,
   WockyNode *node;
   WockyStanzaSubType sub_type;
   const gchar *from, *to, *id;
+  WockyContact *contact;
 
   g_return_val_if_fail (iq != NULL, NULL);
 
@@ -433,6 +506,11 @@ create_iq_reply (WockyStanza *iq,
       sub_type_reply, to, from, ap);
 
   wocky_node_set_attribute (wocky_stanza_get_top_node (reply), "id", id);
+
+  contact = wocky_stanza_get_from_contact (iq);
+  if (contact != NULL)
+    wocky_stanza_set_to_contact (reply, contact);
+
   return reply;
 }
 
@@ -462,7 +540,7 @@ wocky_stanza_build_iq_result_va (
  * wocky_stanza_build_iq_error:
  * @iq: a stanza of type #WOCKY_STANZA_TYPE_IQ and sub-type either
  *      #WOCKY_STANZA_SUB_TYPE_SET or #WOCKY_STANZA_SUB_TYPE_GET
- * @Varargs: a wocky_stanza_build() specification
+ * @...: a wocky_stanza_build() specification
  *
  * Builds an error reply to @iq containing the given body. This function also
  * adds the child element of @iq to the reply, as recommended by <ulink
@@ -668,3 +746,49 @@ wocky_stanza_get_to (WockyStanza *self)
 
   return wocky_node_get_attribute (wocky_stanza_get_top_node (self), "to");
 }
+
+WockyContact *
+wocky_stanza_get_to_contact (WockyStanza *self)
+{
+  g_return_val_if_fail (self != NULL, NULL);
+  g_return_val_if_fail (WOCKY_IS_STANZA (self), NULL);
+
+  return self->priv->to_contact;
+}
+
+WockyContact *
+wocky_stanza_get_from_contact (WockyStanza *self)
+{
+  g_return_val_if_fail (self != NULL, NULL);
+  g_return_val_if_fail (WOCKY_IS_STANZA (self), NULL);
+
+  return self->priv->from_contact;
+}
+
+void
+wocky_stanza_set_to_contact (WockyStanza *self,
+    WockyContact *contact)
+{
+  g_return_if_fail (self != NULL);
+  g_return_if_fail (WOCKY_IS_STANZA (self));
+  g_return_if_fail (WOCKY_IS_CONTACT (contact));
+
+  if (self->priv->to_contact != NULL)
+    g_object_unref (self->priv->to_contact);
+
+  self->priv->to_contact = g_object_ref (contact);
+}
+
+void
+wocky_stanza_set_from_contact (WockyStanza *self,
+    WockyContact *contact)
+{
+  g_return_if_fail (self != NULL);
+  g_return_if_fail (WOCKY_IS_STANZA (self));
+  g_return_if_fail (WOCKY_IS_CONTACT (contact));
+
+  if (self->priv->from_contact != NULL)
+    g_object_unref (self->priv->from_contact);
+
+  self->priv->from_contact = g_object_ref (contact);
+}
