@@ -1,6 +1,11 @@
 from file_transfer_helper import SendFileTest, FileTransferTest, \
     ReceiveFileTest, exec_file_transfer_test
 
+from config import JINGLE_FILE_TRANSFER_ENABLED
+
+if not JINGLE_FILE_TRANSFER_ENABLED:
+    print "NOTE: built with --disable-file-transfer or --disable-voip"
+    raise SystemExit(77)
 
 class SendFileBeforeAccept(SendFileTest):
     def __init__(self, file, address_type,
