@@ -77,3 +77,5 @@ PRIVACY = 'jabber:iq:privacy'
 INVISIBLE = 'urn:xmpp:invisible:0'
 GOOGLE_SHARED_STATUS = 'google:shared-status'
 VERSION = 'jabber:iq:version'
+TP_FT_METADATA_SERVICE = 'http://telepathy.freedesktop.org/xmpp/file-transfer-service'
+TP_FT_METADATA = 'http://telepathy.freedesktop.org/xmpp/file-transfer-metadata'
